import { Service } from "../lib/service"

import { MemoryLockService } from "../service/memoryLockService"
import { MongodbService } from "../service/mongodbService"
import { ExpressService } from "../service/expressService"
import { RouteSessionService } from "../service/routeSessionService"
import { AccountService } from "../service/accountService"
import { AuthenticationService } from "../service/authenticationService"
import { TTLSessionStorage } from "../service/ttlSessionStorage"
import { IncrementalIdService } from "../service/incrementalIdService"


import { UploadService } from "../service/uploadService"
import { BlobService } from "../service/blobService"

export type AllServices = {
    MemoryLockService: MemoryLockService
    MongodbService: MongodbService
    ExpressService: ExpressService
    RouteSessionService: RouteSessionService
    AccountService: AccountService
    AuthenticationService: AuthenticationService
    TTLSessionStorage: TTLSessionStorage
    IncrementalIdService: IncrementalIdService
    UploadService: UploadService
    BlobService: BlobService
}



export abstract class KeyValueStorage extends Service<KeyValueStorage.Settings> {
    constructor(settings: KeyValueStorage.Settings) {
        super({
            kvs: {
                collection: "kvs"
            }
        }, settings)
    }
    services: {
        MongodbService: MongodbService
    }
    readonly name = "KeyValueStorage"
    readonly dependencies = ["MongodbService"]
    abstract set<T>(k: string, v: T, callback?: Callback<null>)
    abstract get<T>(k: string, callback: Callback<T>)
}
