type Callback<T, E = Error> = (err?: E, data?: T) => void
type GUID = string
declare namespace AccountService {
    export type AccountId = GUID
    interface AccountDefinition<TProp = any, TRole = string> {
        id: string
        name: string
        displayName: string
        role?: TRole
        props?: TProp
    }
    type Account = AccountDefinition<any, string>
}

declare namespace AuthenticationService {
    export type Settings = {
        authentication?: {
            ttl?: number
        }
    }
    export interface AuthenticationData<TType extends string = string, TMeta = any> {
        // We use uuid to identify unique authentication
        // suggest using hash of the unique criteria from TMeta
        guid?: string
        accountId?: string
        createdAt?: number
        active?: boolean
        type: TType
        meta: TMeta
    }
    export type CommonAuthenticationData = AuthenticationData<"common", {
        name: string
        displayName: string
        email: string
        password?: string
        passwordHash?: string
        active: boolean
    }>
}

declare namespace KeyValueStorage {
    export type Settings = {
        kvs: {
            collection: string
        }
    }
}

declare namespace BlobService {
    export type Settings = {
        blob: {
            route?: string
            storage: string
        }
    }
}

declare namespace ExpressService {
    export type Settings = {
        http?: {
            port?: number
            host?: string
            offline?: boolean
            limit: {
                jsonInKb?: number
                bodyInKb?: number
            }
            staticResources?: {
                path: string
                route: string
            }[]
        }
    }
}

declare namespace RouteSessionService {
    export type Settings = {
        http?: {
            session?: {
                secret?: string
                maxAge?: number
                resave?: boolean
                saveUninitialized?: boolean
            }
        }
    }
}
declare namespace WechatService {
    export type Settings = {
        wechat: {
            appId: string
            secret: string
            domain?: {
                filename: string
                content: string
            }
            // used for redirect back from wechat
            // should contain the domain
            baseUrl: string
        }
    }
}
